#include "InputLexer.h"
#include "Parsodus/inputparser.h"

namespace pds {

    InputParserException::InputParserException(std::string what): m_what(what) {} 
    const char* InputParserException::what() const throw() {
            return m_what.c_str();
    }

    Config InputParser::parseInput(std::istream& is) {
        InputLexer lex(is);
        Config config;


        bool readingTerminals = false;
        bool readingGrammar = false;
        
        try {
            while(true) {
                InputLexer::Token token = lex.nextToken();
                if(token.type != InputLexer::TERMINAL)
                    readingTerminals = false;
                if(token.type != InputLexer::TERMINAL 
                        && token.type != InputLexer::ARROW 
                        && token.type != InputLexer::VARIABLE 
                        && token.type != InputLexer::SEMICOLON 
                        && token.type != InputLexer::PIPE)
                    readingGrammar = false;
                switch(token.type) {
                    case InputLexer::PARSER:
                        lexColon(lex, token);
                        if(token.type == InputLexer::PARSERTYPE) {
                            if(token.content == "lalr(1)") {
                                config.parserType = Config::ParserType::LALR_1;
                            } else
                                throw InputParserException("Unkown parser type");
                        } else {
                            throw InputParserException("inputfile malformed, no parser type found in parser section");
                        }
                        break;
                    case InputLexer::LEXESIS:
                        lexColon(lex, token);
                        if(token.type != InputLexer::LEXESISNAME)
                            throw InputParserException("No valid input file found after 'lexesis', found '" + token.content + "' instead.");
                        config.lexesisFile = token.content;
                        break;
                    case InputLexer::TERMINALS:
                        lexColon(lex, token, false);
                        readingTerminals = true;
                        break;
                    case InputLexer::GRAMMAR:
                        lexColon(lex, token, false);
                        readingGrammar = true;
                        break;
                    case InputLexer::TERMINAL:
                        if(readingTerminals) {
                            config.grammar.terminals.insert(token.content);
                        } else {
                            throw InputParserException("Found a terminal outside a grammar or terminals section: " + token.content);
                        }
                        break;
                    case InputLexer::START:
                        lexColon(lex, token);
                        if(token.type != InputLexer::VARIABLE)
                            throw InputParserException("Expected to find a start variable, but found: " + token.content);
                        config.grammar.start = token.content;
                        break;
                    case InputLexer::VARIABLE:
                        if(readingGrammar) {
                            if(config.grammar.variables.find(token.content) == config.grammar.variables.end())
                                config.grammar.variables.insert(token.content);
                           
                            std::string current_head = token.content;
                            std::set<Rule> current_rules;
                            if(config.grammar.rules.count(current_head)) {
                                current_rules = config.grammar.rules[current_head];
                            }
                            // Parsing rule
                            token = lex.nextToken();
                            if(token.type != InputLexer::ARROW) 
                                throw InputParserException("No arrow found after '"+ current_head+"', but found '" + token.content + "' instead");
                            Rule rule;
                            rule.head = current_head;
                            bool parsing_head = true;
                            while(parsing_head) {
                                token = lex.nextToken();
                                switch(token.type) {
                                    case InputLexer::VARIABLE:
                                        rule.tail.push_back(token.content);
                                        break;
                                    case InputLexer::TERMINAL:
                                        rule.tail.push_back(token.content);
                                        break;
                                    case InputLexer::SEMICOLON:
                                        parsing_head = false;
                                    case InputLexer::PIPE:
                                        rule.tail.shrink_to_fit();
                                        current_rules.insert(rule);
                                        rule.tail.clear();
                                        break;
                                    default:
                                        throw InputParserException("Expecting to find a variable, terminal, pipe or a semicolon, but found '" + token.content + "' instead");
                                }
                            }
                            config.grammar.rules[current_head] = current_rules;
                        } else
                            throw InputParserException("Found a variable outside a grammar section: " + token.content);
                        break;
                    default: 
                        break;
                    
                }
            }
        } catch(InputLexer::NoMoreTokens& err) {

        }
        return config;
    }
    void InputParser::lexColon(InputLexer& lex, InputLexer::Token &token, bool nextoken) {
        token = lex.nextToken();
        if(token.type != InputLexer::COLON) 
            throw InputParserException("No colon found before '" + token.content + "'");
        if(nextoken)
            token = lex.nextToken();

    }

}
